"""
Description: 进度条的实现
version:
Author: TianyuYuan
Date: 2021-04-02 23:56:05
LastEditors: TianyuYuan
LastEditTime: 2021-04-06 21:25:10
"""
from datetime import datetime

from rich import print as rprint


class ProgressBar():
    """类：进度条，可用于显示普通迭代的进度"""

    def __init__(self, task, length, batchs=50):
        """类：进度条，可用于显示迭代的进度
        - task: 任务名称，可填入·函数名·或者·字符串·,若填入函数名，则可自动获取该函数的名字描述
        - length: 任务长度，通常为可迭代对象的长度，即len(iter_files)
        - batch: 进度条的份数，默认为100个单位，以百分制显示进度
        """
        if isinstance(task, str):
            self.task_name = task
        else:
            self.task_name = task.__name__
        self.length = length
        if self.length < batchs:
            self.batchs = self.length
            self.batch_size = 1
        else:
            self.batchs = batchs
            self.batch_size = self.length / self.batchs
        self.begin_time = datetime.now()

    def progress_layout(self):
        # TODO 进度条布局模块化
        layout = ""
        return layout

    def time_countdown(self, progress):
        if progress == 0:
            return "--:--"
        now_time = datetime.now()
        time_cost = now_time - self.begin_time
        time_cost = time_cost.seconds
        predict_time = round(time_cost / progress * self.batchs)
        time_cost = round(time_cost)
        return time_cost, predict_time

    def print_progressbar(self, i):
        """
        - i: 迭代到了第i个job
        """
        progress = int(i / self.batch_size)
        p_bar = "[" + "[bold green]>[/bold green]" * progress + "]"
        p_propotion = "[green]{}[/green]/[red]{}[/red]".format(i, self.length)
        p_percentage = ":rocket: {}%".format(round(i / self.length * 100))
        p_time = "[cyan]{}/{}s[/cyan]".format(*self.time_countdown(progress))
        if i < self.length:
            rprint(f"{self.task_name}  {p_percentage}  {p_time} :{p_bar}  {p_propotion}", end="\r")
        else:
            rprint(f"{self.task_name}  {p_percentage}  {p_time} :{p_bar}  {p_propotion}")



"""
Description: higher api for pb (progress bar).py
version:
Author: TianyuYuan
Date: 2021-03-26 13:44:18
LastEditors: TianyuYuan
LastEditTime: 2021-04-06 21:20:09
"""
from concurrent.futures import ThreadPoolExecutor, as_completed
from functools import partial

from .progressbar import ProgressBar


# * * * * * * * * * * * * * * * * * * * * * * * #
# *        Higher API for progressbar         * #
# * * * * * * * * * * * * * * * * * * * * * * * #

def pb_iter(iter_files):
    """生成器，将可迭代对象填入，在生成element的同时显示迭代的进度"""
    pb = ProgressBar("iter", len(iter_files))
    i = 0
    for element in iter_files:
        i += 1
        pb.print_progressbar(i)
        yield element


def pb_range(*args):
    """可显示迭代进度的range()，功能用法与range相同
    """
    iter_files = range(*args)
    return pb_iter(iter_files)


def pb_multi_thread(workers: int, task, iter_files) -> list:
    """显示多进程进度条
    - workers: 指定多进程的max_workers
    - task: 任务函数
    - iter_files: 填入要处理的可迭代对象
    - return: 返回每个job的结果，并存入list返回
    """
    pb = ProgressBar(task, len(iter_files))
    result = []
    with ThreadPoolExecutor(max_workers=workers) as pool:
        job_list = []
        for task_input in iter_files:
            job = pool.submit(task, task_input)
            job_list.append(job)
        i = 0
        for done_job in as_completed(job_list):
            i += 1
            result.append(done_job.result())
            pb.print_progressbar(i)
    return result


def pb_multi_thread_partial(workers: int, task, iter_files, **kwargs):
    """显示多进程进度条，针对具有多参数的任务
    - workers: 指定多进程的max_workers
    - task: 任务函数
    - iter_files: 填入要处理的可迭代对象
    - **kwargs: 填入'keyword=constant_object....'
    - return: 返回每个job的结果，并存入list返回
    """
    new_task = partial(task, **kwargs)
    new_task.__name__ = task.__name__
    return pb_multi_thread(workers, new_task, iter_files)


# ! * * * * * * * * * * * * * * * * * * * * * * #
# !           Test Cases & Examples           * #
# ! * * * * * * * * * * * * * * * * * * * * * * #
def square_a_num(x):
    """任务函数"""
    import time
    time.sleep(0.05)
    return x * x


def multi_param_task(x, a, b, c):
    """多参数任务函数"""
    return x + a + b + c


def pb_range_testcase(*args):
    result = []
    for i in pb_range(*args):
        result.append(square_a_num(i))
    # print(result)


def pb_simple_iter_testcase(x):
    result = []
    for i in pb_iter(range(x)):
        result.append(square_a_num(i))
    # print(result)


def pb_multi_thread_testcase(x):
    iter_files = range(x)
    result = pb_multi_thread(10, square_a_num, iter_files)
    # print(result)


def pb_multi_thread_partial_testcase(x, a, b, c):
    iter_files = range(x)
    result = pb_multi_thread_partial(10, multi_param_task, iter_files, a=a, b=b, c=c)
    # print(result)



'''
Description: 解析np_samples
version:
Author: TianyuYuan
Date: 2021-04-06 21:22:57
LastEditors: TianyuYuan
LastEditTime: 2021-04-15 15:08:46
'''
import json
import os.path as osp


class ParseNP:
    """
    ## 解析标注结果(np_samples.json)工具包
    """

    @staticmethod
    def read_json(json_path) -> dict:
        """读取json文件"""
        with open(json_path, 'r') as f:
            data = json.load(f)
        return data

    @staticmethod
    def get_ids2sample(data) -> dict:
        """
        描述：获取以ids为key，所对应sample为val的dict
        - data: 由json导入后的np_samples.json
        - return: ids2sample{'ids':sample}
        - ⚠️注意：key是ids，与图片名不通用
        """
        ids2sample = {}
        for sample in data['images']:
            for ids in sample['ids']:
                ids2sample[ids] = sample
        return ids2sample

    @staticmethod
    def get_request2sample(data) -> dict:
        """
        描述：获取以request为key，所对应sample为val的dict
        - data: 由json导入后的np_samples.json
        - return: request2sample={'request':sample}
        - ⚠️注意：key是图片名，带.jpg后缀
        """
        request2sample = {}
        for sample in data['images']:
            rqsts = sample['request_images']
            for rqst in rqsts:
                rqst = osp.basename(rqst)
                request2sample[rqst] = sample
        return request2sample

    @staticmethod
    def get_request2index(data:dict) -> dict:
        """
        描述：获取以request为key，所对应index为val的dict
        - data: 由json导入后的np_samples.json
        - return: request2index={'request':sample_index}
        - ⚠️注意：key是图片名，带.jpg后缀
        """
        request2index = {}
        for index, sample in enumerate(data['images']):
            requests = sample['request_images']
            for rqst in requests:
                rqst = osp.basename(rqst)
                request2index[rqst] = index
        return request2index

    @staticmethod
    def get_register2sample(data) -> dict:
        '''
        描述：获取以register为key，所对应sample为val的dict
        - data: 由json导入后的np_samples.json
        - return: register2sample{'register':sample}
        - ⚠️注意：key是图片名，带.jpg后缀
        '''
        register2sample = {}
        for sample in data['images']:
            registers = sample['register_images']
            for register in registers:
                register = osp.basename(register)
                register2sample[register] = sample
        return register2sample

    @staticmethod
    def get_register2index(data) -> dict:
        """从data中获得register和index的关系，方便后续用index直接修改p_data"""
        register2index = {}
        for index, sample in enumerate(data['images']):
            registers = sample['register_images']
            for register in registers:
                register = osp.basename(register)
                register2index[register] = index
        return register2index

    @staticmethod
    def get_ids2index(data: dict) -> dict:
        """从data中获得ids和index的关系，方便后续用index直接修改p_data"""
        ids_index = {}
        for index, sample in enumerate(data['images']):
            ids = sample['ids'][0]
            ids_index[ids] = index
            index += 1
        return ids_index
    
    @staticmethod
    def total_samples(data) -> int:
        """统计data中有多少个sample"""
        if not isinstance(data, dict):
            # not dict, is data's path?
            data = ParseNP.read_json(data)
        return len(data['images'])

    @staticmethod
    def total_requests(data) -> int:
        """
        描述：统计data中有多少的request_images
        - data: 由json导入后的np_samples.json
        - return: request_images的总数
        """
        if not isinstance(data, dict):
            # not dict, is data's path?
            data = ParseNP.read_json(data)
        total = 0
        for sample in data['images']:
            total += len(sample['request_images'])
        return total

    @staticmethod
    def total_registers(data) -> int:
        """
        描述：统计data中有多少的register_images
        - data: 由json导入后的np_samples.json
        - return: register_images的总数
        """
        if not isinstance(data, dict):
            # not dict, is data's path?
            data = ParseNP.read_json(data)
        total = 0
        for sample in data['images']:
            total += len(sample['register_images'])
        return total

    @staticmethod
    def show_info(data, np=""):
        """展示np_samples的主要信息"""
        if not isinstance(data, dict):
            data = ParseNP.read_json(data)
        total_samples = ParseNP.total_samples(data)
        total_registers = ParseNP.total_registers(data)
        total_requests = ParseNP.total_requests(data)
        if np == "p":
            print("p_samples的信息：")
        if np == "n":
            print("n_samples的信息")
        print("sample总数为：", total_samples)
        print("register总数为：", total_registers)
        print("request总数为：", total_requests)

if __name__ == "__main__":
    print(help(ParseNP))


"""
Description: 类，针对facex的客户端，可以发送请求返回应答，不同考虑请求格式
version:
Author: TianyuYuan
Date: 2021-02-07 11:54:35
LastEditors: TianyuYuan
LastEditTime: 2021-03-23 19:36:45
"""
import base64
import json
import os
import os.path as osp

import numpy as np
import requests

__DNS_AIBEE = {
    'gpu204': "172.16.10.4",
    'gpu205': "172.16.10.5",
    'gpu206': "172.16.10.6",
}

class FacexClient():
    """
    针对facex的客户端，可以发送请求返回应答，不用考虑请求格式
    """

    def __init__(self, ip, port, group_id='FacexClient-tyyuan'):
        """
        初始化一个facex的客户端
        - ip: 服务器的ip地址(ip可直接选择输入gpu204/gpu205/gpu206)
        - port: facex的服务端口
        - group_id: 注册照片的组名
        """
        if ip in __DNS_AIBEE:
            ip = __DNS_AIBEE[ip]
        self.url = "http://{}:{}".format(ip, port)
        self.api_add = "/users/v1/add"
        self.api_compare_1v1 = "/face/v1/compare-two-image"
        self.api_compare_1vn = "/face/v1/compare-1vn"
        self.api_get_feature = "/face/v1/get-feature"
        self.api_process = '/face/v1/process'
        self.group_id = group_id

    @staticmethod
    def img2base64(img_address: str) -> str:
        """读取图片地址，并将图片编码为base64"""
        with open(img_address, "rb") as fp:
            encoded_string = base64.b64encode(fp.read()).decode('utf-8').replace("'", "")
        return encoded_string

    def add(self, img_address, user_id, check=False) -> dict:
        """注册服务
        - img_address: abs path of the img
        - check: the swith of quality check
        - user_id
        - return: response
        """
        img64 = self.img2base64(img_address)
        data = {"user": {"user_id": user_id, "image": img64}, "groups": [self.group_id], "check": check}
        data = json.dumps(data)
        r = requests.post(self.url + self.api_add, data=data)
        return r.json()

    def add_samples(self, sample: dict, to_register: str, to_id: str, check: bool) -> dict:
        """注册sample
        - sample: 样本字典
        - to_register: 注册request_images or register_images
        - to_id: 以什么为user_id: img_name(without .jpg) or group_name
        - check: the swith of quality check
        - return: the response dict
        """
        img_address = sample[to_register][0]
        if to_id == "name":
            user_id = os.path.basename(img_address)[:-4]
        else:
            print("Unsupported user_id!")
            exit()
        r = self.add(img_address, user_id, check)
        return r

    @staticmethod
    def open_dataset(dataset) -> dict:
        """打开数据集"""
        if os.path.exists(dataset):
            with open(dataset, 'r') as fp:
                data = json.load(fp)
        elif isinstance(dataset, dict):
            data = dataset
        else:
            print("Unsupported dataset type, please make sure the input type is path or dict")
            exit(1)
        return data

    def add_dataset(self, dataset, to_register="register_images", to_id="name", check=False, worker=10):
        """注册整个数据集，适用于注册一个n_samples.json or p_samples.json
        - dataset: 数据集的字典
        - to_register: 注册request_images or register_images
        - to_id: 以什么为user_id: img_name(without .jpg) or group_name
        - check: the swith of quality check
        - worker: 多线程的大小
        - return log_list: 返回注册记录列表
        """
        from functools import partial
        from concurrent.futures import ThreadPoolExecutor
        executor = ThreadPoolExecutor(max_workers=worker)
        data = self.open_dataset(dataset)
        sample_list = data['images']
        log_list = list(
            executor.map(partial(self.add_samples, to_register=to_register, to_id=to_id, check=check), sample_list))
        return log_list

    def compare_1v1(self, img1_address, img2_address, threshold=80, check=False) -> bool:
        """比对1v1
        - img1_address,img2_address: abs path of the imgs
        - threshold: 阈值，大于此阈值则返回True
        - check: the switch of quality check
        - return: True: the same person
        """
        img1 = self.img2base64(img1_address)
        img2 = self.img2base64(img2_address)
        data = {"image1": img1, "image2": img2, "check": check}
        data = json.dumps(data)
        r = requests.post(self.url + self.api_compare_1v1, data=data)
        if r.json()["data"]["score"] < threshold:
            return False
        else:
            return True

    def compare_1vn(self, img_address, top_n=1, check=False, op=31) -> dict:
        """比对1vn
        - img_address: abs path of the img
        - top_n: 返回的最相似的个数
        - check: the swith of quality check
        - return: response
        """
        img64 = self.img2base64(img_address)
        data = {"image": img64, "group_id": self.group_id, "top_n": top_n, "check": check, "face_info_op": op}
        data = json.dumps(data)
        r = requests.post(self.url + self.api_compare_1vn, data=data)
        # check
        if r.status_code != requests.codes.ok:
            return None
        r = r.json()
        if r['error_no'] != 0:
            return None
        return r

    def compare_1vn_samples(self, sample, to_compare, top_n, check) -> dict:
        """比对1vn 输入维度为sample
        - sample: 样本
        - to_compare: 拿样本里的什么进行比对：register_images/request_images
        - top_n: 返回前n个结果
        - check: the swith of quality check
        - return: {图片名：对比结果字典}
        """
        img_path = sample[to_compare][0]
        img = os.path.basename(img_path)
        r = self.compare_1vn(img_path, top_n, check)
        return {img: r}

    def compare_1vn_dataset(self, dataset: dict, to_compare="register_images", top_n=1, check=False, worker=10) -> list:
        """比对1vn 输入维度为sample
        - dataset:数据集
        - to_compare: 拿样本里的什么进行比对：register_images/request_images
        - top_n: 返回前n个结果
        - check: the switch of quality check
        - worker: 多线程的大小
        - return: 结果列表[{img_name1:r1},{img_name2:r2}.....]
        """
        from functools import partial
        from concurrent.futures import ThreadPoolExecutor
        executor = ThreadPoolExecutor(max_workers=worker)
        data = self.open_dataset(dataset)
        sample_list = data["images"]
        log_list = list(executor.map(partial(self.compare_1vn_samples, to_compare=to_compare, top_n=top_n, check=check),
                                     sample_list))
        return log_list

    @staticmethod
    def base64tondarray(feature):
        """将64编码的特征转换成512维的特征向量"""
        tmp = base64.b64decode(feature)
        tmp2 = np.frombuffer(tmp, np.float32)
        return tmp2

    def get_feature(self, img_address) -> np.ndarray:
        """抽取图片的特征向量"""
        img64 = self.img2base64(img_address)
        data = {'image': img64}
        r = requests.post(self.url + self.api_get_feature, data=data)
        feature = r.json()['data']['feature_matrix']
        feature_array = self.base64tondarray(feature)
        return feature_array

    @staticmethod
    def write_log(log_list: list, log_name="log"):
        """保存log记录，可用于
        compare_1vn_dataset
        add_dataset之后
        """
        with open(log_name) as fp:
            json.dump(log_list, fp, indent=4)

    def img_process(self, img_address: str, op=31) -> dict:
        """处理图片，获取face_info信息
        - img_address: abs path of the img
        - op: 返回那些信息
        - return: response
        """
        img64 = self.img2base64(img_address)
        data = {'image_base64': img64, 'op': op}
        data = json.dumps(data)
        r = requests.post(self.url + self.api_process, data=data)
        if r.status_code != requests.codes.ok:
            return None
        r = r.json()
        if r['error_no'] != 0:
            return None
        return r

    # -------------------------------- 用facex建立rgbir/rgbdir/rgbd112的数据集 -------------------------------- #
    def get_box(self, img_address: str, check=True) -> dict:
        """从img_process的response中提取检测框的信息
        - img_address: abs path of the img
        - return: a dict contains the info of the box
        """
        if check:
            r = self.compare_1vn(img_address, check=True, op=10)
        else:
            r = self.img_process(img_address, op=10)
        if not r:
            return None
        box = r['data']['face_info']['box']
        return box

    def get_box_txt(self, txt_address: str, img_type=1) -> dict:
        """以txt为单位的提取图片检测框信息
        - txt_address: the abs address of txt: line: rgb (d) (ir)
        - return: ans = {txt_name:{img:box,img:box,...},txt_name:{...},...}
        """
        txt_name = osp.basename(txt_address).split('.')[0]
        ans = {txt_name: {}}
        with open(txt_address, 'r') as fp:
            lines = fp.readlines()
        for line in lines:
            line_list = line.strip().split()
            rgb_path = line_list[img_type]
            box = self.get_box(rgb_path, check=True)
            if not box:
                continue
            ans[txt_name][rgb_path] = box
        return ans

if __name__ == "__main__":
    print(help(FacexClient))


'''
Description: rlog: Rich log. 丰富的日志打印
version:
Author: TianyuYuan
Date: 2021-04-03 00:02:11
LastEditors: TianyuYuan
LastEditTime: 2021-04-06 21:57:32
'''
from rich.console import Console
from rich.panel import Panel

_console = Console()


class RLog:
    """
    ## Rich Log. 丰富的日志打印
    ### method：
    say(line), saynum(line,num), start, done, error, caution, stage
    """

    @staticmethod
    def say(line):
        """简单的log"""
        _console.log(line)

    @staticmethod
    def saynum(line, num):
        """语句+数字变量"""
        line += str(num)
        _console.log(line)

    @staticmethod
    def start(line: str, title: str = 'Start!', align='left', padding=0):
        """提示开始的log"""
        _console.log(
            Panel.fit(
                line,
                title=f'[green bold]{title}[/green bold]',
                border_style='green',
                padding=padding,
            ),
            justify=align
        )

    @staticmethod
    def done(line: str, title: str = 'Completed!', align='left', padding=0):
        """提示完成的log"""
        _console.log(Panel.fit(
            line,
            title=f'[green bold]{title}[/green bold]',
            border_style='green',
            padding=padding,
        ), justify=align)

    @staticmethod
    def error(line: str, title: str = 'Error!', align='left', padding=0):
        """提示错误的log"""
        _console.log(Panel.fit(
            line,
            title=f'[bold red]{title}[/bold red]',
            border_style='red',
            padding=padding,
        ), justify=align)

    @staticmethod
    def caution(line: str, title: str = 'Caution!', align='left', padding=0):
        """提示，注意事项的log"""
        _console.log(Panel.fit(
            line,
            title=f'[bold cyan]{title}[/bold cyan]',
            border_style='cyan',
            padding=padding,
        ), justify=align)

    @staticmethod
    def stage(title: str = 'Next Stage'):
        """打印水平横线外加标题，适用于提示stage change"""
        _console.rule(title)


if __name__ == "__main__":
    # ! RLog test and examples
    rlog = RLog()
    rlog.start('欢迎使用tykit :toolbox:，tykit目前含有丰富的进度条打印方法，以及五颜六色的记录打印方法。\
帮您更好的了解任务进度、使用多进程、展示任务日志等', title='欢迎使用👏', align='center', padding=1)
    rlog.say("您可以这样来展示一个任务或状态的开始：rlog.start('Task one start!',align='center)")
    rlog.start('Task one start!', align='center')
    rlog.say("您也可以打印一条横线，来展示某一阶段的结束: rlog.stage('Stage 1 ended, Stage 2 start!')")
    rlog.stage('Stage 1 ended, Stage 2 start!')
    rlog.caution('您可以用`rlog.caution()`函数输出一些需要注意的消息，提示将以蓝色表示')
    rlog.error('或是用红色来输出一些错误和警告:skull:')
    rlog.stage()
    rlog.say("当然，还包括如何输出这样一段简单的话：rlog.say('当然，还包括如何输出这样一段简单的话：...')")
    rlog.say("又或是调用rlog.saynum来展示文本+变量的提示：")
    rlog.caution("var a = 1;\nrlog.saynum('var a is :',a)", title="rlog.saynum()的用法")
    a = 1
    rlog.saynum('var a is :', a)
    rlog.done("Thanks for reading, hope you enjoy it! :smile:", title="感谢🙏", align='center', padding=1)



'''
Description: 对标注结果np——samples的进一步抽象，以samples.json作为对象，集成删除，显示信息等方法
version: 
Author: TianyuYuan
Date: 2021-04-14 23:00:45
LastEditors: TianyuYuan
LastEditTime: 2021-04-15 15:07:02
'''
from .parse_np import ParseNP
import json
import os.path as osp

class NPsamples():
    """
    ## 对象化np_samples.json
    对标注结果np_samples的进一步抽象，以samples.json作为对象，集成删除，显示信息等方法
    """

    def __init__(self, jsonfile, np:str):
        """
        对象化np_samples.json
        @params: jsonfile 可以是json的地址，或解析后的dict
        @params: np 是n_samples.json就填”n“，反之填”p“
        """
        if isinstance(jsonfile,str):
            self.data = ParseNP.read_json(jsonfile)
        elif isinstance(jsonfile,dict):
            self.data = jsonfile
        else:
            print("Unsupported sample file type: ",jsonfile)
        self.np = np
        self.group_id = self.data["images"][0]["group_id"]
        self.total_samples = ParseNP.total_samples(self.data)
        self.total_requests = ParseNP.total_requests(self.data)
        self.total_registers = ParseNP.total_registers(self.data)

    def request2sample(self) -> dict:
        return ParseNP.get_request2sample(self.data)
    
    def request2index(self) -> dict:
        return ParseNP.get_request2index(self.data)

    def register2sample(self) -> dict:
        return ParseNP.get_register2sample(self.data)

    def register2index(self) -> dict:
        return ParseNP.get_register2index(self.data)
    
    def ids2sample(self) -> dict:
        return ParseNP.get_ids2sample(self.data)

    def ids2index(self) -> dict:
        return ParseNP.get_ids2index(self.data)

    def show_info(self) -> dict:
        ParseNP.show_info(self.data,np=self.np)

    def check_empty_sample(self) -> list:
        """检查是否有空注册照的情况，返回空注册照的sample_list"""
        empty_sample = []
        for sample in self.data['images']:
            register_list = sample['register_images']
            # request_list = sample['request_images']
            # ids = sample['ids'][0]
            # if not request_list:
            #     print(f"{ids} don't have any requests")
            if not register_list:
                empty_sample.append(sample)
        print("Find empty 'register' samples: ", len(empty_sample))
        return empty_sample

    def save_json(self,name="new_p_samples.json"):
        """以name为名保持json，name需要json后缀"""
        # p_samples.json 应该在保持之前检查是否有空注册照的情况
        if self.np == "p":
            empty_sample = self.check_empty_sample()
            if empty_sample:
                self.delete_samples(empty_sample)
        with open(name,"w") as fp:
            json.dump(self.data, fp, indent=4)
        print(f"The {name} has been saved!")
        self.show_info()

    def delete_requests(self, requests:list):
        """
        将requests列表中的request_image从data中删除
        @params: requests 需要删除的request图片的列表，注意列表中的每一项必须是request_path（because list.remove）
        @return: void 副作用，改变了self.data
        """
        print(f"Before deleting, {self.np}_data has requests: {self.total_requests}")
        print("There are {} requests should be deleted.".format(len(requests)))
        request2index = self.request2index()
        for request_path in requests:
            request_name = osp.basename(request_path)
            index = request2index[request_name]
            self.data['images'][index]["request_images"].remove(request_path)
        self.total_requests = ParseNP.total_requests(self.data)
        print(f"After deleting, {self.np}_data has requests: {self.total_requests}")

    def delete_registers(self, registers:list):
        """
        将register列表中的register_image从data中删除
        @params: registers:list 需要删除的register图片的列表，注意列表中的每一项必须是register_path（because list.remove）
        @return: void 副作用，改变了self.data
        """
        print(f"Before deleting, {self.np}_data has registers: {self.total_registers}")
        print("There are {} registers should be deleted.".format(len(registers)))
        register2index = self.request2index()
        for register_path in registers:
            register_name = osp.basename(register_path)
            index = register2index[register_name]
            self.data["images"][index]["register_images"].remove(register_path)
        self.total_registers = ParseNP.total_registers(self.data)
        print(f"After deleting, {self.np}_data has registers: {self.total_registers}")

    def delete_samples_by_registers(self,registers:list):
        """
        将包含registers的sample从data中删除
        @params: registers:list 需要删除的register图片的列表，注意列表中的每一项必须是register_path（because list.remove）
        @return: void 副作用，改变了self.data
        """
        print(f"Before deleting, {self.np}_data has samples: {self.total_samples}")
        register2sample = self.request2sample()
        dele_sample_list = []
        for register_path in registers:
            register_name = osp.basename(register_path)
            sample = register2sample[register_name]
            dele_sample_list.append(sample)
        dele_set = set(dele_sample_list)
        print("There are {} samples should be deleted".format(len(dele_set)))
        for sample in dele_set:
            self.data["images"].remove(sample)
        self.total_samples = ParseNP.total_samples(self.data)
        print(f"After deleting, {self.np}_data has samples: {self.total_samples}")

    def delete_samples(self, samples:list):
        """
        将samples列表中的sample从data中删除
        @params: samples:list 需要删除的samples列表
        @return: void Side-effect: self.data
        """
        print(f"Before deleting, {self.np}_data has samples: {self.total_samples}")
        dele_set = set(samples)
        print("There are {} samples should be deleted".format(len(dele_set)))
        for sample in dele_set:
            self.data["images"].remove(sample)
        self.total_samples = ParseNP.total_samples(self.data)
        print(f"After deleting, {self.np}_data has samples: {self.total_samples}")

    def __new_sample(self,ids:list,requests:list,registers:list) -> dict:
        sample = {
            "ids":ids,
            "group_id":self.group_id,
            "request_images":requests,
            "register_images":registers
        }
        return sample

    def merge_samples(self,sample1,sample2):
        """
        融合两个sample
        @params sample1
        @params sample2
        @return: void
        """
        ids_list1 = sample1["ids"]
        requests1 = sample1["request_images"]
        registers1 = sample1["register_images"]
        ids_list2 = sample2["ids"]
        requests2 = sample2["request_images"]
        registers2 = sample2["register_images"]
        new_ids = ids_list1 + ids_list2
        new_requests = requests1 + requests2
        new_registers = registers1 + registers2
        new_sample = self.__new_sample(new_ids, new_requests, new_registers)
        self.data["images"].remove(sample1)
        self.data["images"].remove(sample2)
        self.data["images"].append(new_sample)
        
    def add_request_to_sample(self, request_path:str, index:int):
        """
        将request_path添加到index位的sample里
        @params: request_path 要添加的request的绝对地址
        @params: index 目的地sample的index
        @return: void
        """
        self.data["images"][index].append(request_path)


if __name__ == "__main__":
    print(help(NPsamples))


'''
Description: 
version: 
Author: TianyuYuan
Date: 2021-04-02 15:41:57
LastEditors: TianyuYuan
LastEditTime: 2021-04-15 15:09:24
'''
import setuptools

with open("README.md", "r") as fh:
  long_description = fh.read()

setuptools.setup(
  name="tykit",
  version="0.1.1",
  author="tyyuan",
  author_email="1374736649@qq.com",
  description="A tool kit of progress bars and console logs with rich output, also has a bunch of useful functions used in work.",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/paperplane110/tykit",
  packages=setuptools.find_packages(),
  install_requires=['rich>=9'],
  classifiers=[
  "Programming Language :: Python :: 3",
  "License :: OSI Approved :: MIT License",
  "Operating System :: OS Independent",
  ],
)


'''
Description: tykit(TianYu Kit)
version:
Author: TianyuYuan
Date: 2021-04-02 15:40:39
LastEditors: TianyuYuan
LastEditTime: 2021-04-14 23:48:49
'''
name = "tykit"
from .pb_api import pb_range, pb_iter, pb_multi_thread, pb_multi_thread_partial
from .progressbar import ProgressBar
from .rlog import RLog


# * * * * * * * * * * * * * * * * * * * * * * * #
# * * * * * * Exclusive for AIBEE * * * * * * * #
# * * * * * * * * * * * * * * * * * * * * * * * #

from .facex_client import FacexClient   # 方便使用face-x服务
from .parse_np import ParseNP           # 解析np_samples.json的方法库
from .npsamples import NPsamples        # 对标注结果np_samples的进一步抽象，以samples.json作为对象，集成删除，显示信息等方法


if __name__ == "__main__":
    rlog.start('hello world')
